﻿namespace ParametricIteratorMethods {

	class A {
		public int x;

		public IEnumerable<int> Range(int from, int to) {
			if (to < from) {
				throw new ArgumentException($"Value of '{nameof(from)}' has to be less or equal than value of '{nameof(to)}'.");
			}

			return iteratorImplementation(from, to, this);

			static IEnumerable<int> iteratorImplementation(int from, int to, A sourceA) {
				Console.WriteLine("+ A.Range: before 1st yield return");
				from--;
				to++;
				for (int i = from; i <= to; i++) {
					yield return sourceA.x + i;
				}
				sourceA.x++;
				Console.WriteLine("+ A.Range: after last yield return");
			}
		}
	}

	class Program {
		static void Main(string[] args) {
			A a = new A();

			Console.WriteLine("Main: trying to create Range from 10 to 1");
			var range = a.Range(10, 1);
			Console.WriteLine("Main: SUCCESS: created Range from 10 to 1");

			PrintAllMultipleTimes(range);
		}

		static void PrintAllMultipleTimes<T>(IEnumerable<T> items) {
			for (int i = 0; i < 2; i++) {
				Console.WriteLine($"PrintAllMultipleTimes: before foreach #{i}");
				foreach (var v in items) {
					Console.WriteLine("PrintAllMultipleTimes: got {0}", v);
				}
				Console.WriteLine($"PrintAllMultipleTimes: after foreach #{i}");
			}
		}
	}
}
